/* flash.c
 * This is the main routine for the ATTINY2313 that goes in the dance
 * floor.  It's job is to interpret data and turn that data into
 * color values.
 *
 * Data comes in two byte chunks, like so:
 *
 * byte 1						 byte 2
 * 7 6 5 4 3 2 1 0 | 7 6 5 4 3 2 1 0
 * R R R R G G G G   B B B B A A A A
 * 
 * R = red
 * B = blue
 * G = green
 * A = alpha, could be either amber or white
 *
 * Each chip controlls 4 LEDS in a "cell".
 * 16 cells make a 2' x 2' "tile", and the tiles are ganged together to
 * form the entire floor.
 * 
 * There are no address bytes, so the cells are addressed like a bunch of
 * dominoes.  Each cell has a variable "skip" that determines whether it
 * should interpret the data, or simply pass it along to the next cell.
 * "skip" is set to 1 until the cell receives data, at which point it will
 * pass data along until it receives a reset condition byte (0xAA).
*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "iocompat.h"
#include <inttypes.h>

void ioinit(void) {
	/* PD2, PD3, PD4, PD6 are outputs */
	DDRB |= (1<<PB4)|(1<<PB3)|(1<<PB2);
	DDRD |= (1<<PD2)|(1<<PD3)|(1<<PD4)|(1<<PD5);
	PORTB |= (1<<PD2)|(1<<PD3)|(1<<PD4)|(1<<PD5);
	PORTD |= (1<<PD5);

	PORTB = (1<<PB7)|(1<<PB6);
	
	/* Input Capture Register, sets the peroid of the PWM cycle for timer1,
	 * timer2 is already 8 bit so no worries.  */
	ICR1=0xFF;

	OCR1A=0x7F; OCR1B=0x7F; OCR0A=0x7F; OCR0B=0x7F;
	
	/* I really don't know what to say here.  To get all these flags right
	 * you're going to have to carefully RTFM. */
  TCCR1A = (1<<COM1A1)|(1<<COM1A0)|(1<<COM1B1)|(1<<COM1B0)|(1<<WGM11); 
  TCCR1B = (0<<WGM13)|(0<<WGM12)|(1<<CS10);  

	TCCR0A |= (1<<COM0A1)|(1<<COM0A0)|(1<<COM0B1)|(1<<COM0B0)|(1<<WGM01)|(1<<WGM00);
	TCCR0B |= (0<<CS02)|(0<<CS01)|(1<<CS00);
}

void USART_Init(unsigned int baud) {
  /* Set baud rate */
  UBRRH = (unsigned char)(baud>>8);
  UBRRL = (unsigned char)baud;
  /* Enable receiver and transmitter */
  UCSRB = (1<<RXEN)|(1<<TXEN);
  /* Set frame format: 8data, 1stop bit */
  UCSRC = (1<<UCSZ1)|(1<<UCSZ0);
	/* double the speed */
	UCSRA = (1<<U2X);
}

void USART_Transmit( unsigned char data ) {
  /* Wait for empty transmit buffer */
  while ( !( UCSRA & (1<<UDRE)) ) ;
  /* Put data into buffer, sends the data */
  UDR = data;
}

unsigned char USART_Receive( void ) {
  /* Wait for data to be received */
  while ( !(UCSRA & (1<<RXC)) ) ;
  /* Get and return received data from buffer */
  return UDR;
}

int main(void) {
	uint8_t rxbyte,skip;
	ioinit ();
	USART_Init(0);
	skip=1;
	for(;;){ 
    rxbyte=USART_Receive();    
    /* Reset byte.  Set skip to 1 and pass the reset byte along */ 
    if(rxbyte==0xAA){ 
      skip=1; 
      USART_Transmit(0xAA); 
      continue; 
    } 
    if(skip){ 
      /* One thing to note here is that for some reason OCR0A comes out 
       * inverted.  I'm sure there's some flag to set to make it normal, 
       * but this really is easier */ 
      OCR0A = ~rxbyte;
      OCR0B = ~(USART_Receive());
      OCR1A = ~(USART_Receive());
      OCR1B = ~(USART_Receive());
      skip=0;
      } 
    else{ USART_Transmit(rxbyte); } 
  }
	return (0);
}
